@@ -15,6 +15,9 @@ import advifswlib.util.cacher as cacher
 import advifswlib.util.config as config
 import advifswlib.util.locale as locale
 
+class InterfaceState():
+  off, on, connected, connecting, disconnecting = range(5)
+
 class AbstractInterface():
 
   def __init__(self, bus=None, cb_status_area=None, 
@@ -108,9 +111,11 @@ class AbstractInterface():
   def __destroy(self, button=None):
     self.__destroy_signals()
     self.__destroy_interfaces()
+    #print '%s (%s): destroyed' % (self.__name, self.__class__)
 
   def __init_signals(self):
-    for key in self.state['dbus']: self.__init_signal(key)
+    for key in self.state['dbus']:
+      self.__init_signal(key)
 
   def __init_signal(self, x):
     try:
@@ -122,21 +127,31 @@ class AbstractInterface():
                                                             dbus_interface=self.state['dbus'][x]['iface'], 
                                                             path=self.state['dbus'][x]['path'], 
                                                             **self.state['dbus'][x]['args'])
+      return True
     except:
-      pass
+      return False
 
   def reinit_signal(self, x):
     self.__destroy_signal(x)
     self.__init_signal(x)
 
   def __destroy_signals(self):
-    for key in self.state['dbus']: self.__destroy_signal(key)
+    for key in self.state['dbus']:
+      self.__destroy_signal(key)
 
   def __destroy_signal(self, x):
-    receiver_match = self.state['dbus'][x]['match']
-    self.__bus.clean_up_signal_match(receiver_match)
-    self.__bus.remove_signal_receiver(receiver_match)
-    receiver_match.remove()
+    if 'match' in self.state['dbus'][x]:
+      receiver_match = self.state['dbus'][x].pop('match')
+      #self.__bus.clean_up_signal_match(receiver_match)
+      self.__bus.remove_signal_receiver(receiver_match,
+                                        signal_name=self.state['dbus'][x]['name'], 
+                                        dbus_interface=self.state['dbus'][x]['iface'], 
+                                        path=self.state['dbus'][x]['path']
+                                       )
+      receiver_match.remove()
+      #print '%s: destroyed' % (x)
+      return True
+    return False
 
   def __received_signal(self, key, *args, **kwargs):
     #try: 
@@ -163,14 +178,16 @@ class AbstractInterface():
     windex = self.state['value']
     if self.get_config_value('show-popups') and show_banner and self.state['banner'][windex]:
       text = '%s: %s' % \
-             (self.__name, self.__lcl.get_item(self.state['text'][windex]))
+             (self.__name, self.__lcl.get(self.state['text'][windex]))
       if len(self.state['modes']) > 1:
         text = '%s (%s)' % \
                (text, self.__lcl.get_item(self.state['modes'][self.state['mode']]))
       HildonBanner.show_information(self.__button, text)
+    #print self.__name, value, mode, strength
 
   def __init_interfaces(self):
-    for x in self.interfaces: self.__init_interface(x)
+    for x in self.interfaces:
+      self.__init_interface(x)
 
   def __init_interface(self, x, force=False):
     try:
@@ -186,8 +203,11 @@ class AbstractInterface():
                                                           introspect=self.interfaces[x][4], 
                                                           follow_name_owner_changes=True), 
                                               self.interfaces[x][2])
+      return True
     except Exception, e:
-      self.__interfaces[x] = None
+      del self.__interfaces[x]
+      print e
+      return False
 
   def reinit_interface(self, x, force=False):
     self.__destroy_interface(x)
@@ -195,17 +215,22 @@ class AbstractInterface():
 
   def get_interface(self, x):
     try:
-      if self.__interfaces[x] == None: self.__init_interface(x)
+      if not x in self.__interfaces:
+        self.__init_interface(x)
       return self.__interfaces[x]
     except:
       return None
 
   def __destroy_interfaces(self):
-    for x in self.interfaces: self.__destroy_interface(x)
+    for x in self.interfaces:
+      self.__destroy_interface(x)
 
   def __destroy_interface(self, x):
-    try: self.__interfaces[x] = None
-    except: pass
+    try:
+      del self.__interfaces[x]
+      return True
+    except:
+      return False
 
   def __get_status(self):
     self.state['value'], self.state['mode'], self.state['strength'] = self.get_status()
@@ -219,12 +244,12 @@ class AbstractInterface():
     self.__update_ui()
 
   def __update_ui(self):
-    windex = self.state['value']
-    image = self.state['images'][windex]
-    if windex == 2 and self.state['strength'] > 0:
+    value = self.state['value']
+    image = self.state['images'][value]
+    if value == InterfaceState.connected and self.state['strength'] > 0:
       image = self.state['images_st'][self.state['strength']]
     self.__set_si(image)
-    self.__button.set_value(self.__lcl.get_item(self.state['text'][windex]))
+    self.__button.set_value(self.__lcl.get(self.state['text'][value]))
 
   def __button_clicked(self, button=None, user_data=None):
     return self.__switch(self.get_config_value('show-state-dialog'))
@@ -236,8 +261,10 @@ class AbstractInterface():
     if not self.__processing:
       self.__processing = True
       self.start_animation()
-      if show_dialog: gobject.idle_add(self.__show_switch_dialog)
-      else: self.__switch_state(int(self.state['value'] == 0))
+      if show_dialog:
+        gobject.idle_add(self.__show_switch_dialog)
+      else:
+        self.__switch_state(int(self.state['value'] == InterfaceState.off))
       self.__button.get_toplevel().hide()
     return True
 
@@ -246,7 +273,8 @@ class AbstractInterface():
                                  gtk.DIALOG_MODAL | gtk.DIALOG_NO_SEPARATOR, 
                                  (gtk.STOCK_PREFERENCES, gtk.RESPONSE_OK))
     self.state_control = ToggleControlMaemo5(self.__get_state_controls())
-    self.state_control.set_active(self.state['value'])
+
+    self.state_control.set_active(self.__get_state_control_value())
     settings_dialog.vbox.pack_start(self.state_control.get_box(), True)
     height = self.state_control.get_height()
 
@@ -264,15 +292,27 @@ class AbstractInterface():
     self.__processing = False
     response = settings_dialog.run()
     if response == gtk.RESPONSE_DELETE_EVENT:
-      settings_dialog.destroy()
+      #settings_dialog.destroy()
       self.state_control.destroy()
       self.state_control = None
       if self.mode_control:
         self.mode_control.destroy()
         self.mode_control = None
+      settings_dialog.destroy()
+
+  def __get_state_control_value(self):
+    value = self.state['value']
+    if value == InterfaceState.connecting:
+      return InterfaceState.on
+    elif value == InterfaceState.disconnecting:
+      #return InterfaceState.connected
+      return InterfaceState.on
+    elif value == InterfaceState.connected:
+      return InterfaceState.on
+    return value
 
   def __get_state_controls(self):
-    texts = map(self.__lcl.get_item, list(self.state['text']))
+    texts = map(self.__lcl.get, list(self.state['text'][:2]))
     return zip(texts, [self.__turn_state for t in texts])
 
   def __get_mode_controls(self):
@@ -281,22 +321,25 @@ class AbstractInterface():
 
   def __turn_state(self, widget=None, control=None, cstate=None):
     if widget.get_active():
-      if not(cstate): cstate = control.get_active()
+      if not cstate:
+        cstate = control.get_active()
       chstate = min(1, int(cstate))
-      if chstate != self.state['value']:
+      value = self.__get_state_control_value()
+      if chstate != value:
         try:
           self.__switch_state(chstate)
         except:
           self.switched()
           self.__update_ui()
-      elif cstate != self.state['value']: 
-        control.set_active(self.state['value'])
+      elif cstate != value: 
+        control.set_active(value)
 
   def __turn_mode(self, widget=None, control=None, cmode=None):
     if widget.get_active():
-      if not(cmode): cmode = control.get_active()
+      if not(cmode):
+        cmode = control.get_active()
       chmode = max(0, control.get_active())
-      if self.state['value'] and  chmode != self.state['mode']:
+      if self.state['value'] != InterfaceState.off and chmode != self.state['mode']:
         try:
           self.__switch_mode(chmode)
         except:
@@ -311,8 +354,10 @@ class AbstractInterface():
     if new_state != cstate:
       self.__button.set_value('...')
       try:
-        if new_state: self.turn_on()
-        else: self.turn_off()
+        if new_state:
+          self.turn_on()
+        else:
+          self.turn_off()
       except:
         self.switched()
         self.__update_ui()
@@ -347,7 +392,8 @@ class AbstractInterface():
 
   def __set_sai(self, key):
     if any(self.state['show']):
+      #print self.__name, self.state['value'], self.state['show'][self.state['value']], self.get_config_value('show-sa-icon')
       if self.state['show'][self.state['value']] and self.get_config_value('show-sa-icon'):
         self.__set_status_area_icon(self.__name, key)
-      else: 
+      else:
         self.__set_status_area_icon(self.__name, None, False)
