@@ -7,7 +7,7 @@ WLAN interface object
 '''
 
 import dbus, gobject
-from advifswlib.interfaces.Abstract import AbstractInterface
+from advifswlib.interfaces.Abstract import InterfaceState, AbstractInterface
 from advpowcommon.util.execn import *
 
 class WLANInterface(AbstractInterface):
@@ -26,8 +26,8 @@ class WLANInterface(AbstractInterface):
 
   def pre_init(self):
     self.__strength_timeout = None
-    self.autooff = None
-    self.has_be = get_shell_command_output('ls /lib/modules/$(uname -r) | grep -m 1 wl1251') != ''
+    self.__autooff = None
+    self.has_be = get_shell_command_output('ls /lib/modules/current/*.ko | grep -m 1 wl1251') != ''
     self.state = {'name': 'WLAN',
                   'dbus': {'on': {'name': self.wifi_module_signal, 
                                   'iface': self.wifi_module_iface, 
@@ -43,17 +43,19 @@ class WLANInterface(AbstractInterface):
                                           'args': {}}},
                   'images': ('ifsw_statusarea_wlan_off', 
                              'ifsw_statusarea_wlan_signal_0', 
-                             'ifsw_statusarea_wlan_signal_3'),
+                             'ifsw_statusarea_wlan_signal_3',
+                             'ifsw_statusarea_wlan_connecting',
+                             'ifsw_statusarea_wlan_disconnecting'),
                   'images_st': ('ifsw_statusarea_wlan_signal_0', 
                                 'ifsw_statusarea_wlan_signal_1', 
                                 'ifsw_statusarea_wlan_signal_1', 
                                 'ifsw_statusarea_wlan_signal_2', 
                                 'ifsw_statusarea_wlan_signal_3', 
                                 'ifsw_statusarea_wlan_signal_3'),
-                  'text': ('Off', 'On', '<>'),
-                  'show': (True, False, False),
-                  'banner': (True, True, True),
-                  'value': 0,
+                  'text': ['Off', 'On', '<>', 'Connecting', 'Disconnecting'],
+                  'show': (True, True, False, False, True),
+                  'banner': (True, True, True, True, True),
+                  'value': InterfaceState.off,
                   'strength': 0,
                   'modes': ('wlan-stock',),
                   'mode': 0}
@@ -69,52 +71,68 @@ class WLANInterface(AbstractInterface):
     self.set_config_option('disconnected-timeout', (-1,10,30,60,300))
 
   def process_signal_on(self, *args, **kwargs):
+    result = None
     if args[0] == '/org/freedesktop/Hal/devices/platform_wl12xx':
-      state = int(kwargs['member'] == 'DeviceAdded')
-      if state and self.get_config_value('show-connection-dialog'):
-        #self.scan_request()
-        gobject.idle_add(self.show_connection_dialog)
-        #self.show_connection_dialog()
-      return state, 0, None
-    else: return None, None, None
+      if kwargs['member'] == 'DeviceAdded':
+        result = InterfaceState.on
+        if self.get_config_value('show-connection-dialog'):
+          #self.scan_request()
+          gobject.idle_add(self.show_connection_dialog)
+          #self.show_connection_dialog()
+      elif kwargs['member'] == 'DeviceRemoved':
+        result = InterfaceState.off
+        self.__cancel_autooff()
+        self.__cancel_get_strength()
+    return result, None, None
 
   def process_signal_connected(self, *args):
+    result = None
     if len(args) == 8 and args[3].startswith('WLAN_'):
       if args[7] == 2:
-        if self.autooff:
-          gobject.source_remove(self.autooff)
-          self.autooff = None
+        result = InterfaceState.connected
+        self.__cancel_autooff()
         if self.get_config_value('show-signal-strength'):
           self.__strength_timeout = gobject.timeout_add(1000, self.get_strength)
-        return 2, 0, None
-      elif args[7] == 0 and self.state['value'] != 0:
-        autooff = int(self.get_config_value('disconnected-timeout'))
-        if autooff > 0 and self.state['value'] == 2:
-          self.autooff = gobject.timeout_add(autooff*1000, self.turn_off)
-          gobject.source_remove(self.__strength_timeout)
-        return 1, 0, None
+        ap_bytename = args[5]
+        apname = ''.join([str(char) for char in ap_bytename if char])
+      elif args[7] == 0 and self.state['value'] != 0: #State: On
+        result = InterfaceState.on
+        if self.state['value'] == InterfaceState.disconnecting:
+          #schedule autoff if needed
+          autooff = int(self.get_config_value('disconnected-timeout'))
+          if autooff > 0:
+            self.__autooff = gobject.timeout_add(autooff*1000, self.turn_off)
+      elif args[7] in (1,15) and self.state['value'] != InterfaceState.off:
+        #result = InterfaceState.on
+        result = InterfaceState.connecting
+        self.__cancel_autooff()
+      elif args[7] == 3 and self.state['value'] == InterfaceState.connected:
+        #result = InterfaceState.on
+        result = InterfaceState.disconnecting
+        self.__cancel_get_strength()
       else:
-        return self.state['value'], 0, None
-    else:
-      return None, None, None
+        result = self.state['value']
+    return result, None, None
 
   def process_signal_statistics(self, *args):
     return None, None, min(int(args[7]), 5)
 
   def get_status(self):
-    result = int(get_shell_command_output('lsmod | grep -m 1 wl12xx') != '')
-    if result:
+    loaded = get_shell_command_output('lsmod | grep -m 1 wl12xx') != ''
+    if loaded:
       try:
         self.get_interface('ICD').state_req()
       except:
         self.reinit_interface('ICD')
-    return result, 0, 0
+      return InterfaceState.on, 0, 0
+    else:
+      return InterfaceState.off, 0, 0
 
   def turn_on(self):
-    if self.state['value'] == 0:
+    if self.state['value'] == InterfaceState.off:
       #run_async(self.run_commands, (self.wifi_turn_on_cmd,))
       gobject.idle_add(self.run_commands, self.wifi_turn_on_cmd)
-    elif self.state['value'] == 2:
+    elif self.state['value'] == InterfaceState.connected:
       try:
         self.get_interface('ICD').disconnect_req(dbus.UInt32(0x8000))
       except:
@@ -122,9 +140,9 @@ class WLANInterface(AbstractInterface):
     return False
 
   def turn_off(self):
-    self.autooff = None
-    if self.state['value'] > 0:
-      if self.state['value'] == 2:
+    self.__cancel_autooff()
+    if self.state['value'] != InterfaceState.off:
+      if self.state['value'] == InterfaceState.connected:
         try:
           self.get_interface('ICD').disconnect_req(dbus.UInt32(0x8000))
         except:
@@ -152,10 +170,20 @@ class WLANInterface(AbstractInterface):
       self.reinit_interface('ICD')
 
   def get_strength(self):
-    if self.state['value'] == 2 and self.get_config_value('show-signal-strength'):
+    if self.state['value'] == InterfaceState.connected and self.get_config_value('show-signal-strength'):
       self.get_interface('ICD').statistics_req()
       self.__strength_timeout = gobject.timeout_add(5000, self.get_strength)
     return False
 
+  def __cancel_get_strength(self):
+    if self.__strength_timeout:
+      gobject.source_remove(self.__strength_timeout)
+      self.__strength_timeout = None
+
   def turn_mode(self, mode):
     self.switched()
+
+  def __cancel_autooff(self):
+    if self.__autooff:
+      gobject.source_remove(self.__autooff)
+      self.__autooff = None
