@@ -38,4 +38,7 @@ if __name__ == "__main__":
     print 'Showing...'
     obj.show_all()
     print 'Loaded. Starting gtk.main()'
-    gtk.main()
+    try:
+      gtk.main()
+    except (KeyboardInterrupt, SystemExit):
+      obj.destroy()
